<?php
/**
 * Simple configurable data filter.
 * Note that it is not perfect but should provide a good starting point for
 * filtering input.
 *
 * LICENSE: LGPLv3+
 *
 * Example:
 * <code> 
 * $f = new XSSFilter(True, True, False, False, True);
 * $safer = $f->filter($_GET);
 *
 * $f = new XSSFilter(True, True, False, False, True);
 * $result = $f->filter($input);
 * if ($result->changes) {
 *     print('Your input has been modified');
 * }
 * return $result->data;
 * </code>
 *
 * @author Steve 'Ashcrow' Milner
 * @author Lisa Lu
 * @category security
 * @license LGPLv3+
 * @link http://bitbucket.org/ashcrow/php-xssfilter
 * @package XSSFilter
 * @version 1.0.0
 */


/**
 * Array wrapper for safe arrays.
 */
class ExtendedResponseArray extends ArrayObject {

    /**
     * Class variable noting if the data had been changed.
     * @var bool
     */
    private $changes;

    /**
     * Extended response of filtered array data.
     * @param data array input array which is already safe.
     * @param changes bool if changes had to be made.
     */
    function __construct(Array $data, $changes)
    {
        $this->changes = $changes;
        foreach($data as $key => $value) {
            $this->offsetSet($key, $value);
        }
    }

    /**
     * Converts the instance to a String.
     */
    function __toString()
    {
        return 'Array';
    }

    /**
     * Used to return data requested.
     * @param property string requested attribute.
     */
    function __get($property)
    {
        if ($property == 'changes') {
            return $this->changes;
        } elseif ($property == 'data') {
            return $this->getArrayCopy();
        }
    }
}


/**
 * String wrapper for safer strings.
 */
class ExtendedResponseString {

    /**
     * Class variable noting if the data had been changed.
     * @var bool
     */
    private $changes;
    /**
     * Class variable holding the data directly.
     * @var string
     */
    private $data;

    /**
     * Extended response of filtered string data.
     * @param string data input array which is already safe.
     * @param bool change if changes had to be made.
     */
    function __construct($data, $changes)
    {
        $this->data = $data;
        $this->changes = $changes;
    }

    /**
     * Converts instance to a String.
     * @return string
     */
    function __toString()
    {
        return (string) $this->data;
    }

    /**
     * Used to return data requested.
     * @param string property requested attribute.
     * @return string
     */
    function __get($property)
    {
        if ($property == 'changes') {
            return $this->changes;
        } elseif ($property == 'data') {
            return $this->__toString();
        }
    }
}


/**
 * Configurable XSSFilter for use by developers.
 */
class XSSFilter {

    /**
     * Class variable noting if html should be removed.
     * @var bool
     */
    public $remove_html;
    /**
     * Class variable noting if javascript should be removed.
     * @var bool
     */
    public $remove_javascript;
    /**
     * Class variable noting if html should be escaped.
     * @var bool
     */
    public $escape_html;
    /**
     * Class variable noting if html should be urlencoded.
     * @var bool
     */
    public $urlencode_html;
    /**
     * Class variable noting if the return value should be a structure
     * which includes the scrubbed data and a boolean noting if the data
     * was modified or not.
     * A return of False means no changes, True means changes.
     * @var bool
     */
    public $extended_return;


    /**
     * A filter object that attempts to strip out evil data.
     * @param bool remove_html if html should be removed.
     * @param bool remove_javascript if javascript should be removed.
     * @param bool escape_html if html should be escaped.
     * @param bool urlencode_html if html should be urlencoded.
     * @param bool extended_return if an extended object should be used.
     */
    function __construct($remove_html=False, $remove_javascript=False,
        $escape_html=True, $urlencode_html=False, $extended_return=False) {
        $this->remove_html = $remove_html;
        $this->remove_javascript = $remove_javascript;
        $this->escape_html = $escape_html;
        $this->urlencode_html = $urlencode_html;
        $this->extended_return = $extended_return;
    }

    /**
     * Forward method for filtering input.
     * @param string data the data to filter.
     */
    function filter($data) {
        if (is_array($data)) {
            return $this->filter_array($data);
        }
        return $this->filter_string($data);
    }

    /**
     * Filters a single input item
     * @param string data the data to filter.
     * @return string a safer string.
     */
    function filter_string($data) {
        $original_data = $data;
        if ($this->remove_javascript) {
            $data = preg_replace(
                "@<script[^>]*?>.*?</script[^>]*?>@si", "", $data);
        }
        if ($this->remove_html) {
            // The regex should catch it all, but can't hurt to double check it
            $data = strip_tags(preg_replace(
                "@<.*[^>]*?>.*?</.*>@si", "", $data));
        }
        if ($this->escape_html) {
            $data = htmlspecialchars($data);
        }
        if ($this->urlencode_html) {
            $data = urlencode($data);
        }
        if ($this->extended_return) {
            return new ExtendedResponseString(
                $data, $original_data != $data ? True : False);
        }
        return $data;
    }

    /**
     * Filters a single level array.
     * @param array data the data filled array to filter.
     * @return array a safer array.
     */
    function filter_array($data) {
        $safe = array();
        $changes = False;
        foreach ($data as $key => $value) {
            $result = $this->filter($value);
            if ($this->extended_return) {
                $safe[$key] = $result->data;
                $result->changes and $changes = True;
            } else {
                $safe[$key] = $result;
            }
        }
        if ($this->extended_return) {
            return new ExtendedResponseArray($safe, $changes);
        }
        return $safe;
    }
}
?>
